-- This file is part of FS22_Vehicle_Years.
--
-- FS22_Vehicle_Years is free software: you can redistribute it and/or modify it
-- under the terms of the GNU General Public License as published by the Free
-- Software Foundation, either version 3 of the License, or (at your option) any
-- later version.
--
-- FS22_Vehicle_Years is distributed in the hope that it will be useful, but
-- WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
-- FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
-- details.
--
-- You should have received a copy of the GNU General Public License along with
-- FS22_Vehicle_Years. If not, see <https://www.gnu.org/licenses/>.

VehicleYears = {}

local function vyprint(value)
    if value ~= nil then
        print("VehicleYears: " .. value)
    else
        print("VehicleYears: nil")
    end
end

-- Copied and modified from https://stackoverflow.com/a/27028488
local function dump(o)
    if type(o) == 'table' then
        local s = '{ '
        for k, v in pairs(o) do
            if type(k) ~= 'table' then
                if type(k) ~= 'number' then
                    k = '"' .. k .. '"'
                end
                s = s .. '[' .. k .. '] = ' .. tostring(v) .. ','
            end
        end
        return s .. '} '
    else
        return tostring(o)
    end
end

---Returns the year of a vehicle based on category, brand and name
-- The Categories are values such as animalTransport and baleWrappers
-- @return Year of the vehicle if found otherwise nil
local function loadYearFromXML(category, brand, name, isMod)
    local xmlFile = nil
    local year = nil

    if isMod then
        xmlFile = XMLFile.load("vehicleYearsXMLMods", VehicleYears.modXmldatapath)
    else
        xmlFile = XMLFile.load("vehicleYearsXML", VehicleYears.xmldatapath)
    end

    if xmlFile ~= nil then
        -- Lookup iterate on all vehicles where category and brand matches and
        -- compare all names to find the one matching
        xmlFile:iterate(string.format("vehicles.%s.%s.vehicle", category, brand),
            function(_, key)
                if year == nil then
                    local local_name = g_i18n:convertText(xmlFile:getString(key .. ".name"))

                    if VehicleYears.debugEnabled then
                        vyprint(string.format("key: %s", key))
                        vyprint(string.format("name: %s", name))
                        vyprint(string.format("local_name: %s", local_name))
                    end

                    if local_name == name then
                        year = xmlFile:getString(key .. ".year")
                    end
                end
            end)

        if VehicleYears.debugEnabled then
            vyprint(string.format("year: %s", year))
        end

        -- Vehicles with unknown years are stored as "TBD"
        if year == "TBD" then
            -- Tread TBD as nil to not apply the year in-game
            year = nil
        end

        xmlFile:delete()
    end

    return year
end

---Callback for loading the "year" spec
function VehicleYears.loadYear(xmlFile, customEnvironment, baseDir)
    if VehicleYears.debugEnabled then
        vyprint("VehicleYears.loadYear")
    end
    -- This year does not seem to be used or store in any way...
    -- Keep for now in case we figure out a usecase for it
    return nil
end

---Callback for getting the "year" spec
-- This will use the storeItem to lookup the vehicle in our database
-- @return The year of the vehicle if found otherwise nil
function VehicleYears.getYear(storeItem, realItem)
    if VehicleYears.debugEnabled then
        vyprint("VehicleYears.getYear")
        if VehicleYears.verboseDebugEnabled then
            vyprint(dump(storeItem))
            vyprint(dump(storeItem.specs))
        end

        vyprint(string.format("Vehicle: category %s, brand %s, name %s",
                storeItem.categoryName, storeItem.brandNameRaw, storeItem.name))

        vyprint(storeItem.specs.year)
    end

    -- We store the result in the storeItem - If it is non-nil it means that we
    -- have already stored it, and do not need to lookup in the XML file again
    if storeItem.specs.year == nil then
        storeItem.specs.year = loadYearFromXML(storeItem.categoryName,
            storeItem.brandNameRaw, storeItem.name, storeItem.isMod)
    end

    -- If this is nil, it won't show up in the spec list
    return storeItem.specs.year
end

-- Do initialization if not already done - Ensured to only run once
if VehicleYears.modActivated == nil then
    g_gui:loadProfiles(g_currentModDirectory .. "guiProfiles.xml")

    -- Adding year as a spec allows us to show the value when browsing vehicles
    g_storeManager:addSpecType("year", "shopListAttributeIconDate", nil,
        VehicleYears.getYear, "vehicle")

    VehicleYears.modActivated = true
    VehicleYears.debugEnabled = false
    VehicleYears.verboseDebugEnabled = false
    VehicleYears.xmldatapath = g_currentModDirectory .. "data/vehicle_years.xml"
    VehicleYears.modXmldatapath = g_currentModDirectory .. "data/vehicle_years_mods.xml"

    if VehicleYears.debugEnabled then
        vyprint("initialized")
    end
end
